'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _defineProperty = require('@babel/runtime/helpers/defineProperty');
var ramda = require('ramda');
var React = require('react');
var reactRedux = require('react-redux');
var namespaces = require('@redux-syringe/namespaces');
var utils = require('@redux-syringe/utils');
var _objectSpread = require('@babel/runtime/helpers/objectSpread2');
var _objectWithoutProperties = require('@babel/runtime/helpers/objectWithoutProperties');
var _typeof = require('@babel/runtime/helpers/typeof');
var utilsReact = require('@redux-syringe/utils-react');
var hoistNonReactStatics = require('hoist-non-react-statics');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var _defineProperty__default = /*#__PURE__*/_interopDefaultLegacy(_defineProperty);
var React__default = /*#__PURE__*/_interopDefaultLegacy(React);
var _objectSpread__default = /*#__PURE__*/_interopDefaultLegacy(_objectSpread);
var _objectWithoutProperties__default = /*#__PURE__*/_interopDefaultLegacy(_objectWithoutProperties);
var _typeof__default = /*#__PURE__*/_interopDefaultLegacy(_typeof);
var hoistNonReactStatics__default = /*#__PURE__*/_interopDefaultLegacy(hoistNonReactStatics);

var NamespaceContext = /*#__PURE__*/React.createContext({
  isUseNamespaceProvided: false,
  namespaces: {},
  useNamespace: utils.alwaysNull
});

// NOTE: We use `flip(or)` so falsy values won't override existing truthy ones.
var mergeContextValues = ramda.mergeDeepWith(ramda.flip(ramda.or));
var NamespaceProvider = function NamespaceProvider(_ref) {
  var children = _ref.children,
      feature = _ref.feature,
      namespace = _ref.namespace,
      store = _ref.store,
      useNamespace = _ref.useNamespace;
  var context = React.useContext(NamespaceContext);
  var nextContext = React.useMemo(function () {
    return mergeContextValues(context, {
      namespaces: _defineProperty__default["default"]({}, feature !== null && feature !== void 0 ? feature : namespaces.DEFAULT_FEATURE, namespace),
      useNamespace: useNamespace,
      isUseNamespaceProvided: Boolean(useNamespace)
    });
  }, [context, feature, namespace, useNamespace]);
  var providerElement = /*#__PURE__*/React__default["default"].createElement(NamespaceContext.Provider, {
    value: nextContext
  }, children);
  return store ? /*#__PURE__*/React__default["default"].createElement(reactRedux.Provider, {
    store: store
  }, providerElement) : providerElement;
};

var useNamespace = function useNamespace(feature) {
  var _ref, _namespaces;

  var _useContext = React.useContext(NamespaceContext),
      _useContext$namespace = _useContext.namespaces,
      namespaces$1 = _useContext$namespace === void 0 ? {} : _useContext$namespace,
      _useContext$useNamesp = _useContext.useNamespace,
      useExternalNamespace = _useContext$useNamesp === void 0 ? utils.alwaysNull : _useContext$useNamesp;

  var externalNamespace = useExternalNamespace(feature !== null && feature !== void 0 ? feature : namespaces.DEFAULT_FEATURE, namespaces$1);
  return (_ref = (_namespaces = namespaces$1[feature !== null && feature !== void 0 ? feature : namespaces.DEFAULT_FEATURE]) !== null && _namespaces !== void 0 ? _namespaces : externalNamespace) !== null && _ref !== void 0 ? _ref : undefined;
};

var _excluded = ["feature", "namespace"];
var NAMESPACED_CONNECT_PROPS = 'NAMESPACED_CONNECT_PROPS';

var isFunction = function isFunction(value) {
  return typeof value === 'function';
};

var isObject = function isObject(value) {
  return value != null && _typeof__default["default"](value) === 'object';
};

var alwaysEmptyObject = function alwaysEmptyObject() {
  return {};
};

var wrapMapStateToProps = function wrapMapStateToProps(mapStateToProps) {
  return function (state, ownProps) {
    return mapStateToProps ? mapStateToProps(namespaces.getStateByFeatureAndNamespace(ownProps[NAMESPACED_CONNECT_PROPS].feature, ownProps[NAMESPACED_CONNECT_PROPS].namespace, state), ownProps, state) : {};
  };
};

var wrapActionCreator = function wrapActionCreator(wrappedDispatch) {
  return function (actionCreator) {
    return ramda.compose(wrappedDispatch, actionCreator);
  };
};

var throwTypeError = function throwTypeError() {
  throw new TypeError('mapDispatchToProps is not an object or a function');
};

var wrapMapDispatchToProps = function wrapMapDispatchToProps(mapDispatchToProps) {
  return function (dispatch, ownProps) {
    var wrappedDispatch = ramda.o(dispatch, namespaces.defaultNamespace(ownProps[NAMESPACED_CONNECT_PROPS].namespace));
    return ramda.cond([[ramda.isNil, alwaysEmptyObject], [isFunction, ramda.apply(ramda.__, [wrappedDispatch, ownProps])], [isObject, ramda.map(wrapActionCreator(wrappedDispatch))], [ramda.T, throwTypeError]])(mapDispatchToProps);
  };
};

var rawNamespacedConnect = function rawNamespacedConnect(mapStateToProps, mapDispatchToProps, mergeProps, options) {
  return reactRedux.connect(wrapMapStateToProps(mapStateToProps), wrapMapDispatchToProps(mapDispatchToProps), mergeProps, options);
};
/**
 * @deprecated Prefer `useNamespacedSelector` and `useNamespacedDispatch`.
 */


var namespacedConnect = function namespacedConnect(mapStateToProps, mapDispatchToProps, mergeProps) {
  var _ref = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {},
      optionFeature = _ref.feature,
      optionNamespace = _ref.namespace,
      options = _objectWithoutProperties__default["default"](_ref, _excluded);

  return ramda.compose(utilsReact.mapProps(function (props) {
    var _ref2, _ref3;

    var propFeature = props.feature,
        propNamespace = props.namespace;
    var feature = (_ref2 = optionFeature !== null && optionFeature !== void 0 ? optionFeature : propFeature) !== null && _ref2 !== void 0 ? _ref2 : namespaces.DEFAULT_FEATURE;
    var contextNamespace = useNamespace(feature);
    return _objectSpread__default["default"](_objectSpread__default["default"]({}, props), {}, _defineProperty__default["default"]({}, NAMESPACED_CONNECT_PROPS, {
      feature: feature,
      namespace: (_ref3 = optionNamespace !== null && optionNamespace !== void 0 ? optionNamespace : propNamespace) !== null && _ref3 !== void 0 ? _ref3 : contextNamespace
    }));
  }), rawNamespacedConnect(mapStateToProps, mapDispatchToProps, mergeProps, options), utilsReact.mapProps(ramda.dissoc(NAMESPACED_CONNECT_PROPS)));
};

var useNamespacedDispatch = function useNamespacedDispatch() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      optionFeature = _ref.feature,
      optionNamespace = _ref.namespace;

  var dispatch = reactRedux.useDispatch();
  var feature = optionFeature !== null && optionFeature !== void 0 ? optionFeature : namespaces.DEFAULT_FEATURE;
  var contextNamespace = useNamespace(feature);
  var namespace = optionNamespace !== null && optionNamespace !== void 0 ? optionNamespace : contextNamespace;
  return React.useCallback(function (action) {
    return dispatch(namespace ? namespaces.defaultNamespace(namespace, action) : action);
  }, [dispatch, namespace]);
};

var useNamespacedSelector = function useNamespacedSelector(selector, equalityFn) {
  var _ref = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
      optionFeature = _ref.feature,
      optionNamespace = _ref.namespace;

  var feature = optionFeature !== null && optionFeature !== void 0 ? optionFeature : namespaces.DEFAULT_FEATURE;
  var contextNamespace = useNamespace(feature);
  var namespace = optionNamespace !== null && optionNamespace !== void 0 ? optionNamespace : contextNamespace;
  var namespacedSelector = React.useMemo(function () {
    return ramda.o(selector, function (state) {
      return namespace ? namespaces.getStateByFeatureAndNamespace(feature, namespace, state) : undefined;
    });
  }, [selector, feature, namespace]);
  return reactRedux.useSelector(namespacedSelector, equalityFn);
};

var withNamespaceProvider = function withNamespaceProvider(options) {
  return function (NextComponent) {
    var WithNamespaceProvider = function WithNamespaceProvider(props) {
      return (
        /*#__PURE__*/
        // NOTE: `NamespaceProvider` will ignore any unknown props.
        React__default["default"].createElement(NamespaceProvider, ramda.mergeLeft(options, props), /*#__PURE__*/React__default["default"].createElement(NextComponent, props))
      );
    };

    hoistNonReactStatics__default["default"](WithNamespaceProvider, NextComponent);
    WithNamespaceProvider.displayName = "WithNamespaceProvider(".concat(utilsReact.getDisplayName(NextComponent), ")");
    return WithNamespaceProvider;
  };
};

exports.NamespaceContext = NamespaceContext;
exports.NamespaceProvider = NamespaceProvider;
exports.namespacedConnect = namespacedConnect;
exports.useNamespace = useNamespace;
exports.useNamespacedDispatch = useNamespacedDispatch;
exports.useNamespacedSelector = useNamespacedSelector;
exports.withNamespaceProvider = withNamespaceProvider;
